# $Id: Generator.pm 1867 2009-10-06 07:06:50Z wsl $
# $URL: http://svn.zo.oi/wsl/src/libxyzzy-perl/lib/Xyzzy/Generator.pm $

use utf8;
use strict;
use warnings FATAL => 'all';

package Xyzzy::Generator;

use Spiffy -Base;

use DBI;
use XML::LibXML;
use Apache2::Request;

use Xyzzy::Document;

field 'path';
field 'fullpath';
field 'doc';
field 'content';
field 'code';
field 'type';
field 'config';
field 'dbh';
field 'rec';
field 'req';
field 'msie';
field 'cookiejar';

sub db_datasource {
	my $name = $self->db_name;
	my $type = $self->db_type;
	return "dbi:$type:dbname=$name"
}

sub db_type {
	return 'Pg'
}

sub db_name {
	my $class = ref $self;
	die "No database configured for $class\n"
}

sub db_user {
	my $class = ref $self;
	die "No database user configured for $class\n"
}

sub db_pass {
	return ''
}

my $defaultoptions = {
	PrintError => 0,
	RaiseError => 1,
	ShowErrorStatement => 1,
	pg_enable_utf8 => 1
};

sub db_options {
	return $defaultoptions
}

sub db {
	my $db = $self->dbh;
	return $db if defined $db;
	$db = DBI->connect_cached($self->db_datasource, $self->db_user, $self->db_pass, $self->db_options);
	$db->begin_work;
	$self->dbh($db);
	return $db;
}

sub db_commit {
	my $db = $self->dbh;
	return unless defined $db;
	$db->commit;
#	$db->disconnect;
	$self->dbh(undef);
}

sub db_rollback {
	my $db = $self->dbh;
	return unless defined $db;
	$db->rollback;
#	$db->disconnect;
	$self->dbh(undef);
}

sub param {
	my $req = $self->req;
	return undef
		unless defined $req;
	return $req->param(@_);
}

sub jar {
	my $jar = $self->cookiejar;
	unless(defined $jar) {
		$jar = new Apache2::Cookie::Jar($self->rec);
		$self->cookiejar($jar);
	}
	return $jar;
}

sub cookie {
	my $jar = $self->jar;
	return undef
		unless defined $jar;

	my $cookie = $jar->cookies($_[0]);
	return undef
		unless defined $cookie;

	return $cookie->value
}

sub cfg {
	my $config = $self->config;
	return $config if ref $config;
	$config = $config->get($self->rec);
	$self->config($config);
	return $config;
}

sub find_template {
	my $template = shift;
	my $dir = $self->h->cfg->templatedir;
	return "$dir/$template.xml"
}

sub addTemplates {
	my $doc = $self->doc;
	my $dir = $self->cfg->templatedir;
	$doc->addTemplates(map { "$dir/$_.xml" } @_);
}

sub addContent {
	my $doc = $self->doc;
	my $dir = $self->cfg->staticrootdir;
	$doc->addContent(map { "$dir/$_.xml" } @_);
}

sub init {
	my $doc = Xyzzy::Document->xhtml;
	$self->doc($doc)
}

sub setup {
	my $doc = $self->doc;
	$self->addTemplates('base');
}

sub build {
	my $doc = $self->doc;

	my $title = $doc->getElementById('title');
	$title->appendTextNode(ref $self)
		if defined $title;

	my $headline = $doc->getElementById('headline');
	$headline->appendTextNode(ref $self)
		if defined $headline;
}

sub finalize {
	my $doc = $self->doc;

	$doc->xhtml_cleanup;
#	$doc->validate;

	if($self->msie) {
		$self->type('text/html');
		$self->content($doc->toStringHTML);
	} else {
		$self->type('application/xhtml+xml');
		$self->content($doc->toString);
	}
}

sub new {
	my ($rec, $config, $path, $fullpath) = @_;

	my $req = new Apache2::Request($rec);
	$rec->discard_request_body;

	my $ua = $rec->headers_in->{'User-Agent'};
	my $msie = defined $ua && index($ua, 'MSIE') != -1;

	return super(path => $path, fullpath => $fullpath, config => $config, rec => $rec, req => $req, msie => $msie);
}

sub work {
	$self->init;
	$self->setup;
	$self->build;
	$self->finalize;
}

sub generate {
	my $gen = $self->new(@_);

	eval { $gen->work };
	my $err = $@;
	if($err) {
		eval { $gen->db_rollback };
		warn "db_rollback: $@" if $@;
		die $err;
	}

	$gen->db_commit;

	return $gen;
}
